@@ -1,11 +1,11 @@
 import ast
-import fnmatch
 import os
 import sys
 from pathlib import Path
-from typing import Dict, List, Union
+from typing import Dict, List, Union, Tuple
 
 import pathspec
+from pathspec.pathspec import PathSpec
 import toml
 
 type_comments = sys.version_info >= (3, 8)
@@ -65,7 +65,7 @@ def check_function(
             )
 
     # check keyword arguments for type annotations
-    for i, arg in enumerate(function.args.kwonlyargs):
+    for arg in function.args.kwonlyargs:
         if arg.annotation is None:
             function_issues.append(
                 f"Argument '{arg.arg}' of function '{function_name}' has no type annotation"
@@ -127,100 +127,144 @@ def check_file(file: Path) -> List[str]:
     return walk_ast(ast_tree, file_content=file_content)
 
 
+def load_config(verbose: bool) -> Tuple[List[str], List[str], bool]:
+    # try to load config from pyproject.toml file
+    pyproject = os.path.join(os.getcwd(), "pyproject.toml")
+    if not os.path.isfile(pyproject):
+        return [], [], False
+
+    debug_print(verbose, f"Loading {pyproject}")
+    with open(pyproject, "r", encoding="utf-8") as fp:
+        pyproject_config = toml.load(fp)
+
+    if "tool" not in pyproject_config or "pyleft" not in pyproject_config["tool"]:
+        return [], [], False
+
+    debug_print(verbose, "Loading tool.pyleft settings")
+    config = pyproject_config["tool"]["pyleft"]
+
+    # load extra files/dirs
+    if "files" in config:
+        assert isinstance(config["files"], (str, list))
+
+        # accept space separated list
+        if isinstance(config["files"], str):
+            config["files"] = config["files"].split(" ")
+
+        filenames = config["files"]
+    else:
+        filenames = []
+
+    # load exclusions
+    if "exclude" in config:
+        assert isinstance(config["exclude"], (str, list))
+
+        # accept space separated list
+        if isinstance(config["exclude"], str):
+            config["exclude"] = config["exclude"].split(" ")
+
+        exclusions = config["exclude"]
+    else:
+        exclusions = []
+
+    # load skip gitignore setting
+    if "no-gitignore" in config:
+        assert isinstance(config["no-gitignore"], bool)
+        no_gitignore = config["no-gitignore"]
+    else:
+        no_gitignore = False
+
+    return filenames, exclusions, no_gitignore
+
+
+def does_file_match_exclusion(
+    fileobj: Path, specifications: List[Tuple[Path, PathSpec]]
+) -> bool:
+    for spec in specifications:
+        # if the specification is from a directory not a parent to this file, skip
+        if spec[0] not in fileobj.parents:
+            continue
+
+        # if the speficiaction matches this file relative to the specification,
+        # return True
+        if spec[1].match_file(fileobj.relative_to(spec[0])):
+            return True
+
+    # no matches
+    return False
+
+
 def main(
     filenames: List[str],
     exclusions: List[str] = None,
-    skip_gitignore: bool = False,
+    no_gitignore: bool = False,
     verbose: bool = False,
 ) -> Dict[str, List[str]]:
+
+    cwd = Path(os.getcwd())
+
+    # prevent using mutable type
     if exclusions is None:
         exclusions = []
 
-    # try to load config from pyproject.toml file
-    pyproject = os.path.join(os.getcwd(), "pyproject.toml")
-    if os.path.isfile(pyproject):
-        debug_print(verbose, f"Loading {pyproject}")
-        with open(pyproject, "r", encoding="utf-8") as fp:
-            pyproject_config = toml.load(fp)
+    # load config
+    cfg_filenames, cfg_exclusions, cfg_no_gitignore = load_config(verbose)
 
-        if "tool" in pyproject_config and "pyleft" in pyproject_config["tool"]:
-            debug_print(verbose, "Loading tool.pyleft settings")
-            config = pyproject_config["tool"]["pyleft"]
+    # extend cli options
+    filenames.extend(cfg_filenames)
+    exclusions.extend(cfg_exclusions)
+    no_gitignore = no_gitignore or cfg_no_gitignore
 
-            # load extra files/dirs
-            if "files" in config:
-                assert isinstance(config["files"], (str, list))
-
-                # accept space separated list
-                if isinstance(config["files"], str):
-                    config["files"] = config["files"].split(" ")
-
-                filenames.extend(config["files"])
-
-            # load exclusions
-            if "exclude" in config:
-                assert isinstance(config["exclude"], (str, list))
-
-                # accept space separated list
-                if isinstance(config["exclude"], str):
-                    config["exclude"] = config["exclude"].split(" ")
-
-                exclusions.extend(config["exclude"])
-
-            # load skip gitignore setting
-            if "no-gitignore" in config:
-                assert isinstance(config["no-gitignore"], bool)
-                skip_gitignore = config["no-gitignore"]
-
-            # load quiet setting
-            if "quiet" in config:
-                assert isinstance(config["quiet"], bool)
-                quiet = config["quiet"]
-
-            # load verbose setting
-            if "verbose" in config:
-                assert isinstance(config["verbose"], bool)
-                verbose = config["verbose"]
+    # list of matching specifications, with the directory it came from,
+    # and the specification itself
+    specifications: List[Tuple[Path, PathSpec]] = []
 
     # parse exclusions from gitignore
-    if not skip_gitignore:
-        gitignore = os.path.join(os.getcwd(), ".gitignore")
-        if os.path.isfile(gitignore):
-            debug_print(verbose, f"Loading {gitignore}")
+    if not no_gitignore:
+        for gitignore in cwd.glob("**/.gitignore"):
+            debug_print(verbose, f"Loading {gitignore.absolute()}")
             with open(gitignore, "r", encoding="utf-8") as fp:
-                exclusions.extend(fp.readlines())
+                specifications.append(
+                    (
+                        gitignore.parent.absolute(),
+                        pathspec.PathSpec.from_lines(
+                            pathspec.patterns.GitWildMatchPattern, fp.readlines()
+                        ),
+                    )
+                )
 
-    # prepare match spec from all exclusions
-    spec = pathspec.PathSpec.from_lines(
-        pathspec.patterns.GitWildMatchPattern, exclusions
+    # prepare match spec from given exclusions
+    specifications.append(
+        (
+            cwd.absolute(),
+            pathspec.PathSpec.from_lines(
+                pathspec.patterns.GitWildMatchPattern, exclusions
+            ),
+        )
     )
 
     # create data object to hold all result da6a
     all_issues: Dict[str, List[str]] = {}
 
-    # iterate through all files
+    # build a list of all files
+    all_files = []
     for filename in filenames:
-        # if the filename is a directory, recursively walk it
         if os.path.isdir(filename):
-            for sub_file in Path(filename).glob("**/*.py"):
-                # skip files that are in the exclusions list
-                # compute relative filename to parent directory so as to properly
-                # match gitignore and other exclusion patterns
-                if spec.match_file(sub_file.relative_to(filename)):
-                    debug_print(verbose, f"Skipping {sub_file}")
-                    continue
-
-                debug_print(verbose, f"Checking {sub_file}")
-                all_issues[str(sub_file.relative_to(filename))] = check_file(sub_file)
-
-        # if the filename is a file, check only that
+            all_files.extend(Path(filename).glob("**/*.py"))
         elif os.path.isfile(filename):
-            # also make sure file is not in exclusions
-            if spec.match_file(filename):
-                debug_print(verbose, f"Skipping {filename}")
-                continue
+            all_files.append(Path(filename))
 
-            debug_print(verbose, f"Checking {filename}")
-            all_issues[filename] = check_file(Path(filename))
+    all_files = [p.absolute() for p in all_files]
+
+    # go through all files
+    for fileobj in all_files:
+        rel_filename = fileobj.relative_to(cwd)
+
+        if does_file_match_exclusion(fileobj, specifications):
+            debug_print(verbose, f"Skipping {rel_filename}")
+            continue
+
+        debug_print(verbose, f"Checking {rel_filename}")
+        all_issues[rel_filename] = check_file(fileobj)
 
     return all_issues
